#
#
#   Randline.tcl  v0.1 rrl - April 1995
#
#   See the file: Randline.hlp
#

# ------------------------------------------------------------------
#
# Procedure randlineGetLine
#
#   Return the next non-blank or non-commented line from the file found
# at the filePtr argument. Commented lines have the same syntax as
# tcl comment lines.  (Any amount of white-space followed by the #
# character.)
#
#   Arguments:   filePtr - A valid file pointer that was retuned by the
#                          open command.  Note:  The calling proc 
#                          should do the close.
#
#   Returns:     Returns the line read or "" if the end of file was
#                reached.
#
#------------------------------------------------------------------

proc randlineGetLine {filePtr} {

    global channel    
    set charsRead 0
    set line ""
    
    # Loop while not EOF (charsRead == -1) and we have a blank line
    while {$charsRead != -1 && $line == ""} {
        set charsRead [gets $filePtr line]
        # if not EOF
        if {$charsRead != -1} {
            # Let's see if this is a comment line...
            if { [regexp "^( |\t)*#" $line] } {
                # If it is, let's ignore it by forcing the while to be true.
                set line ""
            } 
        } else {
	    # We're at the end of the file.  The while will terminate. Make
	    # sure that we are returning nothing.
	    set line ""
        }
    }

    return $line
}

# ------------------------------------------------------------------
#
# Procedure randlineIsSafeToEval
#
#       This is a _very_ simplistic check to see if a [ or a ] is in
# the string $line.  We're doing this to avoid doing an eval command
# on a line that has a tcl command embedded.  ([exec "rm -rf /"] or
# something like that.)
#
#       In this form, it will reject lines that do not pose a threat
# like: \[blah, blah\].  Until an algorithm that is 100% safe is found,
# it's better to reject all of these types lines, than to take a chance.
#
#   Arguments: line - The string to check.
#
#   Returns: 0 - if it is a dangerous line
#            1 - if it is not dangerous
#
#------------------------------------------------------------------
proc randlineIsSafeToEval {line} {

    # A regex that looks for any [ or ] character.
    if { [regexp {(\[|\])} $line] } {
	return 0
    } else {
	return 1
    }
}

# ------------------------------------------------------------------
#
# Procedure randlineCheckFile
#
#       This proc checks to see if the file exists, is readable and
# is owned by the eggdrop process's owner.
#
#   Arguments: fileName - the name of the file to check.
#              nick     - the nick of the invoker.
#
#   Returns:  0 - Bad File.
#             1 - Good File.
#
#------------------------------------------------------------------
proc randlineCheckFile {fileName nick} {
    global channel botnick
    
    # Let's check the file out...
    # Make sure that it exists, is owned by the id running the bot
    # and is readable.
    if {[file exists $fileName] == 0 || \
        [file owned  $fileName] == 0 || \
        [file readable $fileName] == 0} {
        puterror "randline: access attempt to invalid file. $fileName"
        putserv "NOTICE $nick: Cannot open file $fileName"
        return 0
    }
    return 1
}
    
# ------------------------------------------------------------------
#
# Procedure randlineDoSafeEval
#
#       This proc checks to see if the line passed as an arguement
# is safe to evaluate and if so, evaluates it.  It also catches any
# any error produced during the evaluation.
#
#   Arguments: fileName - the name of the file where line originated.
#              nick     - the nick of the invoker.
#              line     - the line to evaluate.
#
#   Returns:  "" - Bad File.
#             else the line with the variable substitution.
#
#------------------------------------------------------------------
proc randlineDoSafeEval {fileName nick target message line} {

    global channel botnick 
    
    # Set the beep var for replacement in the eval.
    set beep \007
    set nothing ""

    set outLine ""
    
    if { [randlineIsSafeToEval $line] } {
        # We _finally_ do the eval.
	set catchRc [catch {eval set outLine \"$line\"} catchMsg]
	if {$catchRc != 0} {
            puterror "randline: Error in eval rc = $catchRc : $catchMsg"
            puterror "randline: In file: $fileName  Line : $line"
            putserv "NOTICE $nick :Tcl error in file $fileName"
            set outLine ""
        }
    } else {
	puterror "Dangerous line detected in file $fileName.  Returning null"
	puterror $line
	putserv "NOTICE $nick :Cannot finish command.  The line is considered too dangerous to evaluate."
	putserv "NOTICE $nick :$line"
    }
    
    return $outLine
}

# ------------------------------------------------------------------
#
# Procedure randline
#
#       This proc reads a randline formatted text file, picks a line
# at random and then does an eval against it.  The eval will substitute
# all of the valid tcl variables, to create a personalized line.  See the
# file, randline.txt for more information.
#
#   Arguments: fileName - the name of the file to read.
#              nick     - the nick of the invoker.
#              uhost    - The user/host of the invoker.
#              handle   - The handle of the invoker.
#              target   - the value to substitute in for $target in $line
#              message  - the value to substitute in for $message in $line
#
#   Returns:  "" - if a problem was encountered otherwise
#             the personalized string
#
#------------------------------------------------------------------

proc randline {fileName nick uhost handle target message} {
    global channel botnick
    
    # Let's check the file out...
    if {[randlineCheckFile $fileName $nick] == 0} {
        return ""
    }
    
    # Open the file for reading.
    set f [open $fileName r]
    
    # Read the first line as the number of lines in the file.
    set numLines [randlineGetLine $f]
    
    # Check to see if this is a number.
    if {[regexp {^[0-9]+$} $numLines] == 0} {
        # if not, close the file and return nothing.
        close $f
        puterror "randline: File error.  Expected count, got $numLines"
        return ""
    }
        
    # get a random line number.    
    set lineNumber [rand [expr $numLines]]
    set count 0
    
    # read in the next line.  this _should_ be a good data line.
    set line [randlineGetLine $f]
    while {($count < $lineNumber) && ($line != "")} {
        # keep reading until we hit the count or get a read error
	set line [randlineGetLine $f]
        incr count
    }
    close $f
    
    # if the line is blank at this point, the count at the top of the
    # file is wrong!
    if {$line == ""} {
        putserv "NOTICE $nick :Could not execute command.  Wrong count in input file $fileName."
        puterror "randline: Could not execute command.  Wrong count in input file $fileName."
        return
    }
    
    # Let's check to see if this line has a $message variable in it.
    if {$message == "" && [regexp -nocase {\$\{*message\}*} $line]} {
        # This line requires a message, but the user didn't supply one.
        putserv "NOTICE $nick :Could not execute command, it requires a message."
        putserv "NOTICE $nick :$line"
        return
    }
	
    return [randlineDoSafeEval $fileName $nick $target $message "$line"]
}

# ------------------------------------------------------------------
#
# Procedure randlineInit
#
#      This procedure parses up the args, 
#                     checks to see if a password is needed and valid,
#                     checks to see if all required args are present, 
#                     sets the target to a random nick on the channel (if needed)
#                     returns the results.
#
#   Arguments: targetRequired - 0 | 1 - If it should spew an error message
#                                       if no target is supplied.
#              passwordRequired - 0 | 1 - If it should spew an error message
#                                         if no password is supplied or is
#                                         incorrect.
#              nick   - the nick of the invoker.
#              uhost  - the user@host of the invoker.
#              handle - the handle of the invoker.
#              args   - everything else passed into the invoked command.
#
#   Returns:  As a list:
#             
#             target   - The target either passed in or found a random
#             targetWasRandom - Whether or not the target is random.
#             message  - The message parsed from $args.
#
#------------------------------------------------------------------
proc randlineInit {targetRequired passwordRequired nick uhost handle args} {

    global channel botnick

    # Clean up args.  Remove the {}'s
    set args [join $args]
    
    # Init the variables.
    set targetWasRandom 0
    set message ""
    set target ""

    # Do I know this person?
    set thisUser [finduser $nick!$uhost]
    if {$thisUser == "*"} {
        putserv "NOTICE $nick :Sorry, I don't even know you!"
        putserv "NOTICE $nick :/msg $botnick HELP for more information."
        return ""
    }    
    # Is this person banned?
    if {$thisUser == "*ban"} {
        putserv "NOTICE $nick :Sorry, You be banned!!!"
        return ""
    }    
    
    # Do we have any arguments?
    if {$args != ""} {
        # The first word must be the target.
        set target [lindex $args 0]
        # Is a password required?
        if {$passwordRequired == 1} {
                # The 2nd word must be the password.
                set password [lindex $args 1]
                
                # The rest of the line is a message.
        	set message [lrange $args 2 end]
                
                # Is the password valid?
                if {[passwdOk $handle $password] == 0} {
                    putserv "NOTICE $nick :Sorry, a valid password is required."
                    return ""
                }
        } else {
            # Since there is no password, the rest of the line must be
            # a message.
            set message [lrange $args 1 end]
        }
    } else {
        # If a password is required and there are no arguments,
        # return an error.
        if {$passwordRequired == 1} {
           putserv "NOTICE $nick :Sorry, your password is required."
           return ""
        }
    }
    # Was a target passed in?  "." is used as a place holder.  It is
    # not a valid target.
    if { $target == "" || $target == "."} {
        # No target passed in.  Was one required?
	if {$targetRequired} {
	    putserv "NOTICE $nick :Command requires a target nick.  Cannot continue."
	    return ""
	} else {
            # No target passed in and one is not required, so assign
            # one at random from the channel.
	    set chanList [chanlist]
	    set target [lindex $chanList [rand [llength $chanList]]]
            set targetWasRandom 1
	}
    }
    
    # Build the return line as a list.
    lappend returnCode $target $targetWasRandom $message

    return $returnCode
}

# ------------------------------------------------------------------
#
# Procedure randlineCanKick
#
#     Determine if the nick can kick the target.  The bot and all tandem
#  bots are immune from kicking.  Only a master that is not a bot, may kick
#  a master. Unknown nicks can never kick.  Other than that, anything goes.  
#
#     One should bind initiating command to the "o" flag at least.  
#
#   Arguments:  nick   - the nick of the invoker.
#               uhost  - The user@host of the invoker.
#               handle - The handle of the invoker.
#               target - The kickee
#               banFlag - Whether or not this is a ban.
#
#   Returns:  0 - Can't kick
#             1 - Can kick
#
#------------------------------------------------------------------
proc randlineCanKick {nick uhost handle target banFlag} {
    
    global botnick channel

    # Let's log all attempts.
    
    # putlog "cmd_kickRandLine: Attempt to kick $target by $nick $uhost"
    
    # Can't kick myself!
    if {[string compare [string toupper $target] [string toupper $botnick]] == 0} {
	putserv "NOTICE $nick :Sorry, I can't kick myself.  Please try again."
	return 0
    } else {
        # Let's not allow unknown types to kick anyone.
        if {[validuser $handle] == 0} {
            putserv "NOTICE $nick :Sorry, but I don't know you."
            putserv "NOTICE $nick :Please \"/MSG $botnick HELP\" to receive help."
            return 0
        }    

        if {[onchansplit $target] == 1} {
            putserv "NOTICE $nick :Can't kick $target, they are netsplit."
            return 0
        }
        
        if {[onchan $target] == 0} {
            putserv "NOTICE $nick :Can't kick $target, they aren't on the channel!"
            return 0
        }
        
        append targetHandle $target "!" [getchanhost $target]
	set botUser [finduser $targetHandle]
        
        if {$botUser == "*" || $botUser == "*ban"} {
            # This person is either banned or unknown.  Kick away.
            return 1
        }
	
        # Let's not kick a tandem bot.  WARNING: This will only work if
        # the tandem bot is also a user.  After you add the bot, do a
        # ".adduser tandemBotNick"
	if {[matchattr $botUser t]} {
	    putserv "NOTICE $nick :Sorry, I can't kick user $target.  Please try again."
	    return 0
	}
        
	# Am I kicking a master? 
	if {[matchattr $botUser m]} {
	    # Can't ban a master
	    if {$banFlag} {
		putserv "NOTICE $nick :Sorry, a Master can never be banned."
		return 0
	    }
	    # Is the invoker a master?
	    if {[matchattr $handle m] == 0} {
		putserv "NOTICE $nick :Sorry, but only a Master may kick a master."
		return 0
	    }
	    # A bot can't kick a master either.
	    if {[matchattr $handle t] == 1} {
		putserv "NOTICE $nick :Sorry, but bots can't kick a master."
		return 0
	    }
	}
    }
    return 1
}

# ------------------------------------------------------------------
#
# Procedure randlineKick
#
#      Check to see if nick can kick target.  If they can, kick 'em.
#
#             
#   Arguments:  nick    - The nick of the invoker.
#               uhost   - The user@host of the invoker.
#               handle  - The handle of the invoker.
#               target  - The kickee
#               banFlag - Whether or not to ban as well.
#               outLine - The Kick message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc randlineKick {nick uhost handle target banFlag outLine} {
    global botnick channel

    # Can this nick kick target?
    if {[randlineCanKick $nick $uhost $handle $target $banFlag]} {
	if {$banFlag == 0} {
	    putserv "KICK $channel $target :$outLine"
	} else {
	    set banHost [maskhost $target!*[getchanhost $target]] 
	    addpermban $banHost $outLine
	}
    }
}    

proc randlineKick2 {nick uhost handle target banFlag outLine} {
    global botnick channel

    # Can this nick kick target?
    if {[randlineCanKick $nick $uhost $handle $target $banFlag]} {
	if {$banFlag == 0} {
	    putserv "KICK $channel $target :$outLine"
	} else {
	    set banHost [maskhost $target!*[getchanhost $target]] 
	    putserv "MODE $channel -o $target"
	    putserv "KICK $channel $target :$outLine"
	    putserv "MODE $channel +b $banHost"
	}
    }
}    

# ------------------------------------------------------------------
#
# Procedure cmd_sayRandLine 
#
#       To PRIVMSG #channel a random line from fileName.  If target
#   is not specified in the args, a random nick from the channel will
#   be chosen.
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_sayRandLine {fileName passwordRequired nick uhost handle args} {
    
    global channel

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]
 
    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "PRIVMSG $channel :$outLine"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_nickMsgRandLine 
#
#       To PRIVMSG $nick a random line from fileName.   
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_nickMsgRandLine {fileName passwordRequired nick uhost handle args} {

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	if {[onchan $nick] == 0 && $targetWasRandom == 1} {
	    set target $nick
	}
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "PRIVMSG $nick :$outLine"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_targetMsgRandLine 
#
#       To PRIVMSG $target a random line from fileName.   
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_targetMsgRandLine {fileName passwordRequired nick uhost handle args} {

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	if {[onchan $nick] == 0 && $targetWasRandom == 1} {
	    set target $nick
	}
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "PRIVMSG $target :$outLine"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_meRandLine 
#
#       To PRIVMSG #channel :\001ACTION a random line from fileName.   
#   AKA the /me cmd. 
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_meRandLine {fileName passwordRequired nick uhost handle args} {
    global channel

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "PRIVMSG $channel :\001ACTION $outLine\001"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_nickMeRandLine 
#
#       To PRIVMSG $nick :\001ACTION a random line from fileName.   
#   AKA the /me cmd. 
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_nickMeRandLine {fileName passwordRequired nick uhost handle args} {
    global channel

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "PRIVMSG $nick :\001ACTION $outLine\001"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_targetMeRandLine 
#
#       To PRIVMSG $target :\001ACTION a random line from fileName.   
#   AKA the /me cmd. 
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_targetMeRandLine {fileName passwordRequired nick uhost handle args} {
    global channel

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "PRIVMSG $target :\001ACTION $outLine\001"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_noteRandLine 
#
#       To /NOTICE #channel a random line from fileName.   
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_noteRandLine {fileName passwordRequired nick uhost handle args} {
    global channel

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "NOTICE $channel :$outLine"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_nickNoteRandLine 
#
#       To /NOTICE $nick a random line from fileName.   
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_nickNoteRandLine {fileName passwordRequired nick uhost handle args} {
    global channel

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "NOTICE $nick :$outLine"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_targetNoteRandLine 
#
#       To /NOTICE $target a random line from fileName.   
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_targetNoteRandLine {fileName passwordRequired nick uhost handle args} {
    global channel

    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
    
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    putserv "NOTICE $target :$outLine"
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure cmd_kickRandLine 
#
#       To kick the target from the channel with a random message 
#   from fileName.   
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_kickRandLine {fileName passwordRequired nick uhost handle args} {
    global channel botnick
    
    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]

    if {$arg != ""} {
	set target [lindex $arg 0]
        set targetWasRandom [lindex $arg 1]
	set message [join [lrange $arg 2 end]]
	
	set outLine [randline $fileName $nick $uhost $handle $target $message]
	if { $outLine != "" } {
	    randlineKick $nick $uhost $handle $target 0 $outLine
	}
    }
}

# ------------------------------------------------------------------
#
# Procedure  cmd_randGroup
#
#       To pick a random group of action lines from a group file
#  and interpret them.
#  
#       The format of the file is similar to the line file.  Comments
#  are allowed in TCL fashion.  The first non-blank and non-comment
#  line, is the number of groups in the file.  The line of each group
#  is just the word START.  The remainder may be any number of lines.
#  The first word dictates the command to be executed.  Valid commands
#  are:  SAY        - Put the line to the channel
#        NICK_MSG   - Send a /msg to the invoker.
#        TARGET_MSG - Send a /msg to the target.
#        ME         - Action to the channel
#        NICK_ME    - Private action to the invoker
#        TARGET_ME  - Private action to the target
#        KICK       - Kick the target with the line as the message.
#        NOTICE     - Do a channel notice.  /notice #channel
#        NICK_NOTICE   - Do a private notice to the invoker.
#        TARGET_NOTICE - Do a private notice to the target.
#
#   Arguments: fileName         - The File to read.
#              passwordRequired - If a password is required
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#              args             - The rest of the arguments.  Presumably,
#                                 the target and message
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_randGroup {fileName passwordRequired nick uhost handle args} {
    
    global channel botnick
    
    # Do the init.
    set arg [randlineInit 0 $passwordRequired $nick $uhost $handle [join $args]]
    
    # Did the init fail?
    if {$arg == ""} { 
        putserv "NOTICE $nick :Usage fileName \[target\] \[message\]"
        return 
    }
    
    set nothing ""
    set target [lindex $arg 0]
    set targetWasRandom [lindex $arg 1]
    set message [join [lrange $arg 2 end]]

    # Open the file for reading.
    set f [open $fileName r]
    
    # Read the first line as the number of groups in the file.
    set numGroups [randlineGetLine $f]
    
    # Check to see if this is a number.
    if {[regexp {^[0-9]+$} $numGroups] == 0} {
        # if not, close the file and return nothing.
        close $f
        puterror "randgroup: File error.  Expected count, got $numLines"
        return ""
    }
    
    set groupNumber [rand [expr $numGroups]]
    set count 0

    set line [randlineGetLine $f]
    # Spin through the file until we find group number $groupNumber
    while {$count <= $groupNumber && $line != ""} {
        set line [string toupper $line]
        if {$line == "START"} {
            incr count
        }
	set line [randlineGetLine $f]
    }
    
    # If the $line is empty at this point, the count in the file is too high!!
    if {$line == ""} {
        putserv "NOTICE $nick :Could not execute command.  Wrong count in input file $fileName."
        puterror "Could not execute command.  Wrong count in input file $fileName."
        close $f
        return
    } 
    
    set count 0
    set ifCount 0
    set ifIsTrue(0) 0
    
    while {[string toupper $line] != "START" && $line != ""} {

	set firstWord [string toupper [lindex $line 0]]
	set line [lrange $line 1 end]

        # Damn, i wish that i could get a switch to work here!!!
	if {$firstWord == "ENDIF"} {
	    set ifCount [expr $ifCount - 1]
	    set line ""
	} elseif {$firstWord == "ELSE"} { 
	    if {$ifIsTrue($ifCount)} {
		set ifIsTrue($ifCount) 0
	    } else {
		set ifIsTrue($ifCount) 1
	    }
	    set line ""
	} elseif {$firstWord == "IF"} { 
            # Gotta have only three items in the line.
            # This kinda sucks.  You can't do something like:
            # if $message = "Kick Me"
            # like you should be able to.  Oh well, maybe sometime
            # in the future.
            if {[llength $line] != 3} {
                putserv "NOTICE $nick :Illegal IF in file $fileName"
                putserv "NOTICE $nick :IF = $line"
                close $f
                return
            }

            set firstVar [string toupper [randlineDoSafeEval $fileName $nick $target $message [lindex $line 0]]]
            set secondVar [string toupper [randlineDoSafeEval $fileName $nick $target $message [lindex $line 2]]]
            set ifOperator [lindex $line 1]

	    incr ifCount
	    set ifIsTrue($ifCount) 0   

            # I'm not picky about what the operator is.  It's either "="
            # or it's not.  Anything other than "=" is treated as not equal.
            if {$firstVar == $secondVar} {
                if {$ifOperator == "="} {
                   set ifIsTrue($ifCount) 1
                }
            } else {
                if {$ifOperator != "="} {
                   set ifIsTrue($ifCount) 1
                }
            }
	    set line ""
	}
        
        if {$line != ""} {
	    set ifResult 1
            # Are we in any if statements?
	    if {$ifCount != 0} {
                # This if and all of the previous if's must be true.
		set ifIndex 1
		while {$ifIndex <= $ifCount} {
		    if {$ifIsTrue($ifIndex) == 0} {
			set ifResult 0
		    }
		    incr ifIndex
		}
	    }
            # If the if is true, put the line in the array.
            if {$ifResult } {
		set groupLine($count) $line
		set groupWord($count) $firstWord
		incr count
            }   
        }
        # Get the next non-blank non-commented line.
        set line [randlineGetLine $f]
    }
    
    close $f

    # Let's check to see if the resultant group of lines has
    # a message, kick or a ban. 
    
    set hasKick 0
    set hasMsg 0
    set hasBan 0
    set index 0
    set messageRequired 0

    while {$index < $count} {
	switch $groupWord($index) {
	    "TARGET_MSG" { set hasMsg 1  }
	    "KICK"       { set hasKick 1 }
	    "PERMBAN"    { set hasBan 1 }
	    "PERMBAN2"   { set hasBan 1 }
	}
	if {[regexp -nocase {\$\{*message\}*} $groupLine($index)]} {
	    set messageRequired 1
	}
	incr index
    }

    # Require that the target is on the channel.
    if {($hasKick) || ($hasMsg) || ($hasBan)} {
	if {[onchan $target] == 0} {
	    putserv "NOTICE $nick :$target must be on the channel to execute that command."
	    return
	}
    }
    # Is this nick able to kick the target?  
    if {($hasKick) || ($hasBan)} {
	if {[randlineCanKick $nick $uhost $handle $target $hasBan] == 0} {
	    return
	}
    }

    # Is a message required?
    if {($messageRequired) && ($message == "")} {
	putserv "NOTICE $nick :Sorry, a message is required for this command."
	return
    }
    
    # We're ready to evaluate the group.
    set index 0
    while {$index < $count} {
	                                
        set outLine [randlineDoSafeEval $fileName $nick $target $message "$groupLine($index)"]
        if { $outLine != "" } {
	    switch $groupWord($index) {
                "SAY"        { putserv "PRIVMSG $channel :$outLine" }
                "NICK_MSG"   { putserv "PRIVMSG $nick :$outLine" }
                "TARGET_MSG" { putserv "PRIVMSG $target :$outLine" }
                "ME"         { putserv "PRIVMSG $channel :\001ACTION $outLine\001" }
                "NICK_ME"    { putserv "PRIVMSG $nick :\001ACTION $outLine\001" }
                "TARGET_ME"  { putserv "PRIVMSG $target :\001ACTION $outLine\001" }
                "KICK"       { randlineKick $nick $uhost $handle $target 0 $outLine }
                "PERMBAN"    { randlineKick $nick $uhost $handle $target 1 $outLine }
                "PERMBAN2"   { randlineKick2 $nick $uhost $handle $target 1 $outLine }
                "NOTICE"     { putserv "NOTICE $channel :$outLine" }
                "NICK_NOTICE"   { putserv "NOTICE $nick :$outLine" }
                "TARGET_NOTICE" { putserv "NOTICE $target :$outLine" }
                default   { putserv "NOTICE $nick :Unknown command $firstWord in file $fileName" }
	    }     
        } else {
            return
        }
        incr index
    }
    return
}

# ------------------------------------------------------------------
#
# Procedure cmd_joinRandLine
#
#       To either display an help file when new users join the channel
#   or do a channel action of a random line of text from a file for known users.
#
#   Arguments: fileName         - The File to read.
#              helpFileName     - The Help file to send to the new user.
#                                 or "" to disable this feature.
#              autoAddNewUser   - Whether or not to automatically add
#                                 new users.
#              nick             - the nick of the invoker.
#              uhost            - The user@host of the invoker.
#              handle           - The handle of the invoker.
#
#   Returns:  Nada
#
#------------------------------------------------------------------
proc cmd_joinRandLine {fileName helpFileName autoAddNewUser nick uhost handle} {

    global channel botnick admin

    set thisUser [finduser $nick!$uhost]
    if {$thisUser == "*"} {
        if {$autoAddNewUser == 1} {
	   adduser $handle [maskhost $uhost]
        }
        if {$helpFileName != ""} {
	    # Damn, why won't this work?
	    # showhelp $helpFileName $nick 0 0
	    putserv "PRIVMSG $channel :Inside help"
        }   
    } else {
        if {$thisUser == "*ban"} {
            putserv "NOTICE $nick :Banned"
        } else {
	    set outLine [randline $fileName $nick $uhost $handle $nick "No Message"]
	    if {$outLine != ""} {
	        putserv "PRIVMSG $channel :\001ACTION $outLine\001" 
            }    
	}
    }
}
